#Mediation analysis Feb 19 includes p-value calculation for bootstrap and delta method
#The following functions need to be adapted if interactions are to be assessed.
##prob.exp can set to anything when using continuous exposure
#S=exposure, M=mediator one right after S, G=second mediator closest to outcome, Y=outcome

##med.continuous= if want continuous mediator do c(mediator M sd,mediator G sd), else if dichotomous type 0
#exp.continuous= if want continuous exposure do c(mean, sd), else if dichotomous type 0

sim.mediation.data=function(med.continuous,exp.continuous,sample.size,prob.exp, j,  beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s){ 
  #function to generate simulation data, how the mediators and exposures are generated
  #j is just random number to set seed
  #med.continuous= if want continuous mediator do c(mediator M sd,mediator G sd), else if dichotomous type "No"
  #exp.continuous= if want continuous exposure do c(mean, sd), else if dichotomous type "No" 
  #prob.exp can set to anything when using continuous exposure, problematic if used 0.5 since this is a perfect separation for M~S regression
  #S=exposure, M=mediator one right after S, G=second mediator closest to outcome, Y=outcome
  #
  set.seed(j*17+37)
  if(length(med.continuous)==1){
    if (length(exp.continuous)==1){
      S=c(rep(0, (1-prob.exp)*sample.size), rep(1, prob.exp*sample.size))
    }
    else{
      exp.mean=exp.continuous[1]
      exp.sd=exp.continuous[2]
      S=rnorm(sample.size, mean=exp.mean, sd=exp.sd)
    }
    M=rbinom(n=sample.size,size=1, prob=(exp(delta.0+delta.s*S)/(1+exp(delta.0+delta.s*S))))
    G=rbinom(n=sample.size, size=1, prob=exp(alpha.0 + alpha.s*S + alpha.m*M)/(1+exp(alpha.0 + alpha.s*S + alpha.m*M)))
    Y=rbinom(n=sample.size, size=1, prob=exp(beta.0 + beta.s*S + beta.m*M +beta.g*G)/(1+exp(beta.0 + beta.s*S + beta.m*M +beta.g*G)))
    return(cbind(Y,S,M,G))
  }
  
  else{
    sigm=med.continuous[1]
    sigg=med.continuous[2]
    if (length(exp.continuous)==1){#function to generate S, M, G, Y and exposure (S) is dichotomous
      S<-c(rep(0, (1-prob.exp)*sample.size), rep(1, prob.exp*sample.size))
    }
    else {#IF S is continuous to function to generate S, M, G, Y 
      exp.mean=exp.continuous[1]
      exp.sd=exp.continuous[2]
      S<-rnorm(sample.size, mean=exp.mean, sd=exp.sd)
    }
    M=delta.0+delta.s*S+rnorm(sample.size,0, sigm)
    G=alpha.0+alpha.s*S + alpha.m*M + rnorm(sample.size,0,sigg)
    eta=beta.0 + beta.s*S + beta.m *M + beta.g*G + rnorm(sample.size,0,1) 
    prob=pnorm(eta)
    Y=rbinom(n=sample.size, size=1, prob)
    return(cbind(Y,S,M,G))
  }
}

boot.mediation.calc=function(med.continuous, exp.continuous, n.sims, prob.exp, beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s, confounders, sample.size, dist.coeff, n.boot){  
  #to check if 95% CI contains true value 
  #following only used if want to see the coefficient's distribution
  correct.coeff=matrix(NA, nr=1, nc=9)
  confidence.interval.calc=matrix(0, nr=9 ,nc=n.sims) # delta0, delta s .... columns are for each
  true.values=c( beta.0, beta.s ,beta.m, beta.g , alpha.0 ,alpha.s, alpha.m, delta.0,delta.s)
  results1=matrix(NA, nr=n.sims, nc=3)#risk differences
  results2=matrix(NA, nr=n.sims, nc=3)#risk differences
  results3=matrix(NA, nr=n.sims, nc=3)#risk differences
  RR1=matrix(0, nr=n.sims, nc=3) #risk ratio
  RR2=matrix(0, nr=n.sims, nc=3)#risk ratio
  RR3=matrix(0, nr=n.sims, nc=3) #risk ratio
  pathprob1=matrix(NA, nr=n.sims, nc=3) #path probability
  pathprob2=matrix(NA, nr=n.sims, nc=3)
  pathprob3=matrix(NA, nr=n.sims, nc=3)
  pathprob4=matrix(NA, nr=n.sims, nc=3)
  if(length(med.continuous)==1){ #means that we are using 2 dichotomous mediators
    for (i in 1:n.sims){
      sim.data=as.data.frame(sim.mediation.data(med.continuous,exp.continuous, sample.size, prob.exp,i, beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s))
      names(sim.data)=c("Y","S", "M", "G")
      #get point estimates
      outcome.reg=glm(Y~., family = binomial (link="logit"), data=sim.data)
      g.reg=glm(G~.-Y, family = binomial (link="logit"), data=sim.data)
      m.reg=glm(M~.-Y-G, family = binomial (link="logit"), data=sim.data)
      beta.hat=summary(outcome.reg)$coefficients[,1]
      alpha.hat=summary(g.reg)$coefficients[,1]
      delta.hat=summary(m.reg)$coefficients[,1]
      total.effect=summary(glm(Y~.-G-M, family = binomial (link="logit"), data=sim.data))$coefficients[,1]
      total.effect.1=exp(sum(total.effect))/(1+exp(sum(total.effect)))
      total.effect.2=exp(total.effect[1])/(1+exp(total.effect[1]))
      total.rd=total.effect.1-total.effect.2 #when S=1 compared to S=0
      total.rr=total.effect.1/total.effect.2
      
      V.matrix=create.vmatrix(outcome.reg, g.reg, m.reg)#creating covariance matrix 
      
      if (dist.coeff==1){
        x=c(beta.hat[c(1:4)], alpha.hat[c(1:3)], delta.hat[c(1:2)])
        for (s in 1:9){
          if (x[s]-sqrt(V.matrix[s,s])*1.96  <  true.values[s] &  true.values[s] < x[s]+sqrt(V.matrix[s,s])*1.96){
            confidence.interval.calc[s,i]=1}
        }
      }
      
      
      A=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,c()) #first part of difference, confounders=empty vector becuase this none taken account for in simulations
      B=point.var.est.di(0,0,0, beta.hat, alpha.hat, delta.hat,c())
      pathprob1[i,1]=c(B[1]) #prob for path 0,0,0
      pathprob2[i,1]=c(A[1]) #prob for path 1,0,0
      path=A[1]-B[1] #point estimate of probability of this path
      results1[i,1]=path #s->y
      RR1[i,1]=c(A[1]/B[1])
      C=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,c()) #first part of difference
      D=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,c())
      pathprob3[i,1]=c(C[1]) #prob for path 1,1,0
      path2=C[1]-D[1]
      RR2[i,1]=c(C[1]/D[1])
      results2[i,1]=path2 #s->g->y
      E=point.var.est.di(1,1,1, beta.hat, alpha.hat, delta.hat,c()) #first part of difference
      G=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,c())
      pathprob4[i,1]=c(E[1]) #prob for path 1,1,1
      path3=E[1]-G[1]
      RR3[i,1]=c(E[1]/G[1])
      results3[i,1]=path3 #s-> my
      
      var.boot=create.var.boot.di(n.boot, sim.data, c())
      results1[i,c(2:3)]=c(quantile(var.boot[,1], probs=c(0.025, 0.975)))
      results2[i,c(2:3)]=c(quantile(var.boot[,2], probs=c(0.025, 0.975)))
      results3[i,c(2:3)]=c(quantile(var.boot[,3], probs=c(0.025, 0.975)))
      RR1[i,c(2:3)]=c(quantile(var.boot[,4], probs=c(0.025, 0.975)))
      RR2[i,c(2:3)]=c(quantile(var.boot[,5], probs=c(0.025, 0.975)))
      RR3[i,c(2:3)]=c(quantile(var.boot[,6], probs=c(0.025, 0.975)))

    }
    
    #path 1
    sa=1
    sb=0
    sc=0
    
    first=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    sa=0
    second=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    p1=first-second
    r1=first/second
    #path 2
    sa=1
    sb=1
    sc=0
    first=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    sb=0
    second=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    p2=first-second
    r2=first/second
    
    #path 3
    sa=1
    sb=1
    sc=1
    first=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    sc=0
    second=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    p3=first-second
    r3=first/second
    
    p.prob1=mean(pathprob1[,1])
    p.prob2=mean(pathprob2[,1])
    p.prob3=mean(pathprob3[,1])
    p.prob4=mean(pathprob4[,1])
    boot1=check(p1, results1)/n.sims
    boot2=check(p2, results2)/n.sims
    boot3=check(p3, results3)/n.sims
    RRboot1=check(r1, RR1)/n.sims
    RRboot2=check(r2, RR2)/n.sims
    RRboot3=check(r3, RR3)/n.sims
    theoretical.rd=sum(p1, p2, p3)
    theoretical.rr=prod(r1, r2, r3)
    
    # png(filename=paste("bootstrap_", prob.exp, "RiskDifference.png", sep=""))
    # boxplot(cbind(results1[,1]-p1, results2[,1]-p2, results3[,1]-p3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    # png(filename=paste("bootstrap_", prob.exp, "RiskRatio.png", sep=""))
    # boxplot(cbind(RR1[,1]-r1, RR2[,1]-r2, RR3[,1]-r3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    # 
    if (dist.coeff==1){
      correct.coeff=rowSums(confidence.interval.calc)/n.sims
      out.put=c(p1,p2,p3,r1,r2,r3,boot1, boot2, boot3, RRboot1, RRboot2, RRboot3, p.prob1, p.prob2, p.prob3, p.prob4, correct.coeff, total.rd, theoretical.rd, total.rr, theoretical.rr, mean(results1[,1])-p1, sd(results1[,1]), sum((results1[,1]-p1)^2)/n.sims, mean(results2[,1])-p2,sd(results2[,1]), sum((results2[,1]-p2)^2)/n.sims,mean(results3[,1])-p3, sd(results3[,1]), sum((results3[,1]-p3)^2)/n.sims,mean(RR1[,1])-r1, sd(RR1[,1]), sum((RR1[,1]-r1)^2)/n.sims, mean(RR2[,1])-r2,sd(RR2[,1]), sum((RR2[,1]-r2)^2)/n.sims,mean(RR3[,1])-r3,sd(RR3[,1]), sum((RR3[,1]-r3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDboot path 1",  "RDboot path 2", "RDboot path 3","RRboot path 1", "RRboot path 2", "RRboot path 3", "path000", "path100", "path110", "path111","beta.0", "beta.s", "beta.m", "beta.g", "alpha.0", "alpha.s", "alpha.m","delta.0", "delta.s", "total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3" )
      return(out.put)#use this if want to know how well the coefficients were simulated
    }
    else{
      out.put=c(p1,p2,p3,r1,r2,r3,boot1, boot2, boot3, RRboot1, RRboot2, RRboot3 ,p.prob1, p.prob2, p.prob3, p.prob4, total.rd, theoretical.rd, total.rr, theoretical.rr,  mean(results1[,1])-p1, sd(results1[,1]), sum((results1[,1]-p1)^2)/n.sims, mean(results2[,1])-p2,sd(results2[,1]), sum((results2[,1]-p2)^2)/n.sims,mean(results3[,1])-p3, sd(results3[,1]), sum((results3[,1]-p3)^2)/n.sims,mean(RR1[,1])-r1, sd(RR1[,1]), sum((RR1[,1]-r1)^2)/n.sims, mean(RR2[,1])-r2,sd(RR2[,1]), sum((RR2[,1]-r2)^2)/n.sims,mean(RR3[,1])-r3,sd(RR3[,1]), sum((RR3[,1]-r3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDboot path 1", "RDboot path 2", "RDboot path 3","RRboot path 1", "RRboot path 2", "RRboot path 3", "path000", "path100", "path110", "path111","total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3" )
      return(out.put)
    }
  }
  else{ #means that we are using 2 continuous mediators
    #simulations for continuous mediators and dichotomous outcome
    # following is the function to calculate point estimates and variance
    # with sa,sb,sc are the counterfactuals for S, G, M respectively
    for (i in 1:n.sims){
      #make edits as needed
      sim.data=as.data.frame(sim.mediation.data(med.continuous,exp.continuous, sample.size, prob.exp,i, beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s))
      names(sim.data)=c("Y","S", "M", "G")
      outcome.reg=glm(Y~., family = binomial (link="probit"), data=sim.data)
      g.reg=lm(G~.-Y, data=sim.data)
      m.reg=lm(M~.-Y-G, data=sim.data)
      beta.hat=summary(outcome.reg)$coefficients[,1]
      alpha.hat=summary(g.reg)$coefficients[,1]
      delta.hat=summary(m.reg)$coefficients[,1]
      V.matrix=create.vmatrix(outcome.reg, g.reg, m.reg)
      total.effect=summary(glm(Y~.-M-G, family = binomial (link="probit"), data=sim.data))$coefficients[,1]
      total.rd=pnorm(sum(total.effect))-pnorm(sum(total.effect[1])) #when S=1 compared to S=0
      total.rr=pnorm(sum(total.effect))/pnorm(sum(total.effect[1])) 
      #if interested in coefficient simulation values
      if (dist.coeff==1){
        x=c(beta.hat[c(1:4)], alpha.hat[c(1:3)], delta.hat[c(1:2)])
        for (s in 1:9){ # 
          if (x[s]-sqrt(V.matrix[s,s])*1.96  <  true.values[s] &  true.values[s] < x[s]+sqrt(V.matrix[s,s])*1.96){
            confidence.interval.calc[s,i]=1
          }
        }
      }
      
      A=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders) #first part of difference
      B=point.var.est.cont(0,0,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders)
      
      pathprob1[i,1]=c(B[1]) #prob for path 0,0,0
      pathprob2[i,1]=c(A[1]) #prob for path 1,0,0
      
      path=A[1]-B[1]
      results1[i,1]=c(path) #s->y
      RR1[i,1]=c(A[1]/B[1])
      
      C=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders) #first part of difference
      D=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders)
      
      pathprob3[i,1]=c(C[1]) #prob for path 1,1,0
      
      path2=C[1]-D[1]
      results2[i,1]=c(path2) #s-> g-> y
      RR2[i,1]=c(C[1]/D[1])
      
      E=point.var.est.cont(1,1,1, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders) #first part of difference
      G=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg,confounders)
      
      pathprob4[i,1]=c(E[1]) #prob for path 1,1,1
      
      path3=E[1]-G[1]
      results3[i,1]=c(path3) #s-> my
      RR3[i,1]=c(E[1]/G[1])
      var.boot= create.var.boot.cont(n.boot, sim.data, confounders)
      results1[i,c(2:3)]=c(quantile(var.boot[,1], probs=c(0.025, 0.975)))
      results2[i,c(2:3)]=c(quantile(var.boot[,2], probs=c(0.025, 0.975)))
      results3[i,c(2:3)]=c(quantile(var.boot[,3], probs=c(0.025, 0.975)))
      RR1[i,c(2:3)]=c(quantile(var.boot[,4], probs=c(0.025, 0.975)))
      RR2[i,c(2:3)]=c(quantile(var.boot[,5], probs=c(0.025, 0.975)))
      RR3[i,c(2:3)]=c(quantile(var.boot[,6], probs=c(0.025, 0.975)))
    }
    
    
    sa=1
    sb=0
    sc=0
    sigm=med.continuous[1]
    sigg=med.continuous[2]
    s.star=sqrt((sigg^2)*(beta.g^2) + 1 + ((beta.m+beta.g*alpha.m)^2)*(sigm^2))
    
    ###need to make sure the following equations follow the equation specified (esp if interactions/confounders are included)
    first=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    
    sa=0
    second=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    p1=first-second #risk difference number of 95% CI covers
    r1=first/second #risk ratio number of 95% CI covers true
    sa=1
    sb=1
    sc=0
    first=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    sb=0
    second=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    p2=first-second
    r2=first/second
    sa=1
    sb=1
    sc=1
    first=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    sc=0
    second=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    p3=first-second
    r3=first/second
    ##### no more edits required
    ##### 
    p.prob1=mean(pathprob1[,1])
    p.prob2=mean(pathprob2[,1])
    p.prob3=mean(pathprob3[,1])
    p.prob4=mean(pathprob4[,1])
    
    boot1=check(p1, results1)/n.sims
    boot2=check(p2, results2)/n.sims
    boot3=check(p3, results3)/n.sims
    
    RRboot1=check(r1,RR1)/n.sims
    RRboot2=check(r2,RR2)/n.sims
    RRboot3=check(r3,RR3)/n.sims
    theoretical.rd=sum(p1,p2,p3)
    theoretical.rr=prod(r1,r2,r3)
    
    # png(filename=paste("bootstrap_", prob.exp, "RiskDifference.png", sep=""))
    # boxplot(cbind(results1[,1]-p1, results2[,1]-p2, results3[,1]-p3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    # png(filename=paste("bootstrap_", prob.exp, "RiskRatio.png", sep=""))
    # boxplot(cbind(RR1[,1]-r1, RR2[,1]-r2, RR3[,1]-r3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    
    if (dist.coeff==1){
      correct.coeff=rowSums(confidence.interval.calc)/n.sims
      out.put=c(p1,p2,p3,r1,r2,r3,boot1, boot2, boot3, RRboot1, RRboot2, RRboot3, p.prob1, p.prob2, p.prob3, p.prob4, correct.coeff, total.rd, theoretical.rd, total.rr, theoretical.rr,  mean(results1[,1])-p1, sd(results1[,1]), sum((results1[,1]-p1)^2)/n.sims, mean(results2[,1])-p2,sd(results2[,1]), sum((results2[,1]-p2)^2)/n.sims,mean(results3[,1])-p3, sd(results3[,1]), sum((results3[,1]-p3)^2)/n.sims,mean(RR1[,1])-r1, sd(RR1[,1]), sum((RR1[,1]-r1)^2)/n.sims, mean(RR2[,1])-r2,sd(RR2[,1]), sum((RR2[,1]-r2)^2)/n.sims,mean(RR3[,1])-r3,sd(RR3[,1]), sum((RR3[,1]-r3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDboot path 1",  "RDboot path 2", "RDboot path 3","RRboot path 1", "RRboot path 2", "RRboot path 3", "path000", "path100", "path110", "path111","beta.0", "beta.s", "beta.m", "beta.g", "alpha.0", "alpha.s", "alpha.m","delta.0", "delta.s", "total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3"  )
      return(out.put)#use this if want to know how well the coefficients were simulated
    }
    else{
      out.put=c(p1,p2,p3,r1,r2,r3,boot1, boot2, boot3, RRboot1, RRboot2, RRboot3 ,p.prob1, p.prob2, p.prob3, p.prob4, total.rd, theoretical.rd, total.rr, theoretical.rr, mean(results1[,1])-p1, sd(results1[,1]), sum((results1[,1]-p1)^2)/n.sims, mean(results2[,1])-p2,sd(results2[,1]), sum((results2[,1]-p2)^2)/n.sims,mean(results3[,1])-p3, sd(results3[,1]), sum((results3[,1]-p3)^2)/n.sims,mean(RR1[,1])-r1, sd(RR1[,1]), sum((RR1[,1]-r1)^2)/n.sims, mean(RR2[,1])-r2,sd(RR2[,1]), sum((RR2[,1]-r2)^2)/n.sims,mean(RR3[,1])-r3,sd(RR3[,1]), sum((RR3[,1]-r3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDboot path 1", "RDboot path 2", "RDboot path 3","RRboot path 1", "RRboot path 2", "RRboot path 3", "path000", "path100", "path110", "path111","total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3" )
      return(out.put)
    }
  }
}

sim.mediation.calc=function(med.continuous, exp.continuous, n.sims,prob.exp, beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s, confounders, sample.size, dist.coeff){  
  #to check if 95% CI contains true value 
  correct.coeff=matrix(NA, nr=1, nc=9)
  confidence.interval.calc=matrix(0, nr=9 ,nc=n.sims) # delta0, delta s .... columns are for each
  true.values=c( beta.0, beta.s ,beta.m, beta.g , alpha.0 ,alpha.s, alpha.m, delta.0,delta.s)
  results1=matrix(NA, nr=n.sims, nc=3)#risk differences
  results2=matrix(NA, nr=n.sims, nc=3)#risk differences
  results3=matrix(NA, nr=n.sims, nc=3)#risk differences
  RR1=matrix(0, nr=n.sims, nc=3) #risk ratio
  RR2=matrix(0, nr=n.sims, nc=3)#risk ratio
  RR3=matrix(0, nr=n.sims, nc=3) #risk ratio
  pathprob1=matrix(NA, nr=n.sims, nc=3) #path probability
  pathprob2=matrix(NA, nr=n.sims, nc=3)
  pathprob3=matrix(NA, nr=n.sims, nc=3)
  pathprob4=matrix(NA, nr=n.sims, nc=3)
  
  if(length(med.continuous)==1){ #means that we are using 2 dichotomous mediators
    # following is the function to calculate point estimates and variance
    # with sa,sb,sc are the counterfactuals for S, G, M respectively
    for (i in 1:n.sims){
      sim.data=as.data.frame(sim.mediation.data(med.continuous,exp.continuous, sample.size, prob.exp,i, beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s))
      names(sim.data)=c("Y","S", "M", "G")
      #get point estimates
      outcome.reg=glm(Y~., family = binomial (link="logit"), data=sim.data)
      g.reg=glm(G~.-Y, family = binomial (link="logit"), data=sim.data)
      m.reg=glm(M~.-Y-G, family = binomial (link="logit"), data=sim.data)
      beta.hat=summary(outcome.reg)$coefficients[,1]
      alpha.hat=summary(g.reg)$coefficients[,1]
      delta.hat=summary(m.reg)$coefficients[,1]
      total.effect=summary(glm(Y~.-M-G, family = binomial (link="logit"), data=sim.data))$coefficients[,1]
      total.effect.1=exp(sum(total.effect))/(1+exp(sum(total.effect)))
      total.effect.2=exp(total.effect[1])/(1+exp(total.effect[1]))
      total.rd=total.effect.1-total.effect.2 #when S=1 compared to S=0
      total.rr=total.effect.1/total.effect.2
      
      
      
      V.matrix=create.vmatrix(outcome.reg, g.reg, m.reg)#creating covariance matrix 
      
      if (dist.coeff==1){
        x=c(beta.hat[c(1:4)], alpha.hat[c(1:3)], delta.hat[c(1:2)])
        for (s in 1:9){
          if (x[s]-sqrt(V.matrix[s,s])*1.96  <  true.values[s] &  true.values[s] < x[s]+sqrt(V.matrix[s,s])*1.96){
            confidence.interval.calc[s,i]=1
          }
        }
      }
      
      A=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,c()) #first part of difference, confounders=empty vector becuase this none taken account for in simulations
      B=point.var.est.di(0,0,0, beta.hat, alpha.hat, delta.hat,c())
      pathdev1=matrix(B[-1], nrow=1)
      std.error=sqrt(pathdev1%*%V.matrix%*%t(pathdev1))
      pathprob1[i,]=c(B[1], B[1]-1.96*std.error, B[1]+1.96*std.error) #prob for path 0,0,0
      pathdev2=matrix(A[-1], nrow=1)
      std.error=sqrt(pathdev2%*%V.matrix%*%t(pathdev2))
      pathprob2[i,]=c(A[1], A[1]-1.96*std.error, A[1]+1.96*std.error) #prob for path 1,0,0
      
      path=A[1]-B[1] #point estimate of probability of this path
      der=matrix(A[-1]-B[-1], nrow=1)
      std.error=sqrt(der%*%V.matrix%*%t(der))
      results1[i,]=c(path, path-1.96*std.error,path+1.96*std.error) #s->y
      path.rr=A[1]/B[1]
      RRder1=matrix((A[-1]/A[1])-(B[-1]/B[1]),nrow=1)
      RR.std.error=sqrt(RRder1%*%V.matrix%*%t(RRder1))
      RR1[i,]=c(path.rr, exp(log(path.rr)-1.96*RR.std.error) , exp(log(path.rr)+1.96*RR.std.error))
      
      C=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,c()) #first part of difference
      D=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,c())
      pathdev3=matrix(C[-1], nrow=1)
      std.error=sqrt(pathdev3%*%V.matrix%*%t(pathdev3))
      pathprob3[i,]=c(C[1], C[1]-1.96*std.error, C[1]+1.96*std.error) #prob for path 1,1,0
      path2=C[1]-D[1]
      der2=matrix(C[-1]-D[-1], nrow=1)
      std.error2=sqrt(der2%*%V.matrix%*%t(der2))
      results2[i,]=c(path2,path2-1.96*std.error2, path2+1.96*std.error2) #s->g->y
      path2.rr=C[1]/D[1]
      RRder2=matrix((C[-1]/C[1])-(D[-1]/D[1]),nrow=1)
      RR.std.error2=sqrt(RRder2%*%V.matrix%*%t(RRder2))
      RR2[i,]=c(path2.rr, exp(log(path2.rr)-1.96*RR.std.error2) , exp(log(path2.rr)+1.96*RR.std.error2))
      
      E=point.var.est.di(1,1,1, beta.hat, alpha.hat, delta.hat,c()) #first part of difference
      G=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,c())
      pathdev4=matrix(E[-1], nrow=1)
      std.error=sqrt(pathdev4%*%V.matrix%*%t(pathdev4))
      pathprob4[i,]=c(E[1], E[1]-1.96*std.error, E[1]+1.96*std.error) #prob for path 1,1,1
      path3=E[1]-G[1]
      der3=matrix(E[-1]-G[-1], nrow=1)
      std.error3=sqrt(der3%*%V.matrix%*%t(der3))
      results3[i,]=c(path3, path3-1.96*std.error3, path3+1.96*std.error3) #s-> my
      path3.rr=E[1]/G[1]
      RRder3=matrix((E[-1]/E[1])-(G[-1]/G[1]),nrow=1)
      RR.std.error3=sqrt(RRder3%*%V.matrix%*%t(RRder3))
      RR3[i,]=c(path3.rr, exp(log(path3.rr)-1.96*RR.std.error3) , exp(log(path3.rr)+1.96*RR.std.error3)) #s-> my
      
    }
    sa=1
    sb=0
    sc=0
    #first path
    first=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    sa=0
    second=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    true.rd1=first-second
    true.rr1=first/second
    p1=check(true.rd1,results1) #risk difference number of 95% CI covers
    r1=check(true.rr1,RR1) #risk ratio number of 95% CI covers true
    #path 2
    sa=1
    sb=1
    sc=0
    first=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    sb=0
    second=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    true.rd2=first-second
    true.rr2=first/second
    p2=check(true.rd2,results2)
    r2=check(true.rr2, RR2)
    
    #path 3
    sa=1
    sb=1
    sc=1
    first=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    sc=0
    second=dev.comp(G=1,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0,confounders,sa,sb,sc)[1]+dev.comp(G=1,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0 ,confounders,sa,sb,sc)[1]+dev.comp(G=0,M=1,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s, 0, confounders,sa,sb,sc)[1]+dev.comp(G=0,M=0,beta.0,beta.s,beta.m,beta.g,0,alpha.0, alpha.s, alpha.m,0, delta.0,delta.s,0, confounders, sa,sb,sc)[1]
    true.rd3=first-second
    true.rr3=first/second
    p3=check(true.rd3,results3)
    r3=check(true.rr3, RR3)
    
    p.prob1=mean(pathprob1[,1])
    p.prob2=mean(pathprob2[,1])
    p.prob3=mean(pathprob3[,1])
    p.prob4=mean(pathprob4[,1])
    theoretical.rd=sum(true.rd1,true.rd2,true.rd3)
    theoretical.rr=prod(true.rr1, true.rr2, true.rr3)
    # 
    # png(filename=paste("delta_", prob.exp, "RiskDifference.png", sep=""))
    # boxplot(cbind(results1[,1]-true.rd1, results2[,1]-true.rd2, results3[,1]-true.rd3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    # png(filename=paste("delta_", prob.exp, "RiskRatio.png", sep=""))
    # boxplot(cbind(RR1[,1]-true.rr1, RR2[,1]-true.rr2, RR3[,1]-true.rr3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    # 
    if (dist.coeff==1){
      correct.coeff=rowSums(confidence.interval.calc)/n.sims
      out.put=c(true.rd1, true.rd2, true.rd3,true.rr1, true.rr2, true.rr3,p1/n.sims,p2/n.sims,p3/n.sims,r1/n.sims,r2/n.sims,r3/n.sims, p.prob1, p.prob2, p.prob3, p.prob4, correct.coeff, total.rd, theoretical.rd, total.rr, theoretical.rr,mean(results1[,1])-true.rd1, sd(results1[,1]), sum((results1[,1]-true.rd1)^2)/n.sims,mean(results2[,1])-true.rd2, sd(results2[,1]), sum((results2[,1]-true.rd2)^2)/n.sims,mean(results3[,1])-true.rd3,sd(results3[,1]), sum((results3[,1]-true.rd3)^2)/n.sims,mean(RR1[,1])-true.rr1, sd(RR1[,1]), sum((RR1[,1]-true.rr1)^2)/n.sims, mean(RR2[,1])-true.rr2, sd(RR2[,1]), sum((RR2[,1]-true.rr2)^2)/n.sims,mean(RR3[,1])-true.rr3, sd(RR3[,1]), sum((RR3[,1]-true.rr3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDdelta path 1", "RDdelta path 2", "RDdelta path 3","RRdelta path 1", "RRdelta path 2", "RRdelta path 3", "path000", "path100", "path110", "path111","beta.0", "beta.s", "beta.m", "beta.g", "alpha.0", "alpha.s", "alpha.m","delta.0", "delta.s", "total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3" )
      return(out.put)#use this if want to know how well the coefficients were simulated
    }
    else{
      out.put=c(true.rd1, true.rd2, true.rd3,true.rr1, true.rr2, true.rr3,p1/n.sims,p2/n.sims,p3/n.sims,r1/n.sims,r2/n.sims,r3/n.sims,p.prob1, p.prob2, p.prob3, p.prob4, total.rd, theoretical.rd, total.rr, theoretical.rr,mean(results1[,1])-true.rd1, sd(results1[,1]), sum((results1[,1]-true.rd1)^2)/n.sims,mean(results2[,1])-true.rd2, sd(results2[,1]), sum((results2[,1]-true.rd2)^2)/n.sims,mean(results3[,1])-true.rd3,sd(results3[,1]), sum((results3[,1]-true.rd3)^2)/n.sims,mean(RR1[,1])-true.rr1, sd(RR1[,1]), sum((RR1[,1]-true.rr1)^2)/n.sims, mean(RR2[,1])-true.rr2, sd(RR2[,1]), sum((RR2[,1]-true.rr2)^2)/n.sims,mean(RR3[,1])-true.rr3, sd(RR3[,1]), sum((RR3[,1]-true.rr3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDdelta path 1", "RDdelta path 2", "RDdelta path 3","RRdelta path 1", "RRdelta path 2", "RRdelta path 3", "path000", "path100", "path110", "path111","total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3" )
      return(out.put)
    }
    
  }
  else{ #means that we are using 2 continuous mediators
    for (i in 1:n.sims){
      #make edits as needed
      sim.data=as.data.frame(sim.mediation.data(med.continuous,exp.continuous, sample.size, prob.exp,i, beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s))
      names(sim.data)=c("Y","S", "M", "G")
      outcome.reg=glm(Y~., family = binomial (link="probit"), data=sim.data)
      g.reg=lm(G~.-Y, data=sim.data)
      m.reg=lm(M~.-Y-G, data=sim.data)
      beta.hat=summary(outcome.reg)$coefficients[,1]
      alpha.hat=summary(g.reg)$coefficients[,1]
      delta.hat=summary(m.reg)$coefficients[,1]
      V.matrix=create.vmatrix(outcome.reg, g.reg, m.reg)
      total.effect=summary(glm(Y~.-M-G, family = binomial (link="probit"), data=sim.data))$coefficients[,1]
      total.rd=pnorm(sum(total.effect))-pnorm(sum(total.effect[1])) #when S=1 compared to S=0
      total.rr=pnorm(sum(total.effect))/pnorm(sum(total.effect[1])) 
      
      #if interested in coefficient simulation values
      if (dist.coeff==1){
        x=c(beta.hat[c(1:4)], alpha.hat[c(1:3)], delta.hat[c(1:2)])
        for (s in 1:9){ # 
          if (x[s]-sqrt(V.matrix[s,s])*1.96  <  true.values[s] &  true.values[s] < x[s]+sqrt(V.matrix[s,s])*1.96){
            confidence.interval.calc[s,i]=1
          }
          
        }
        
      }
      
      A=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders) #first part of difference
      B=point.var.est.cont(0,0,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders)
      
      pathdev1=matrix(B[-1], nrow=1)
      std.error=sqrt(pathdev1%*%V.matrix%*%t(pathdev1))
      pathprob1[i,]=c(B[1], B[1]-1.96*std.error, B[1]+1.96*std.error) #prob for path 0,0,0
      
      pathdev2=matrix(A[-1], nrow=1)
      std.error=sqrt(pathdev2%*%V.matrix%*%t(pathdev2))
      pathprob2[i,]=c(A[1], A[1]-1.96*std.error, A[1]+1.96*std.error) #prob for path 1,0,0
      
      der=matrix(A[-1]-B[-1], nrow=1)
      std.error=sqrt(der%*%V.matrix%*%t(der))
      path=A[1]-B[1]
      results1[i,]=c(path, path-1.96*std.error,path+1.96*std.error) #s->y
      RRder1=matrix((A[-1]/A[1])-(B[-1]/B[1]),nrow=1)
      RR.std.error=sqrt(RRder1%*%V.matrix%*%t(RRder1))
      RR1[i,]=c(A[1]/B[1], exp(log(A[1]/B[1])-1.96*RR.std.error) , exp(log(A[1]/B[1])+1.96*RR.std.error))
      
      
      C=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg, confounders) #first part of difference
      D=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg, confounders)
      
      pathdev3=matrix(C[-1], nrow=1)
      std.error=sqrt(pathdev3%*%V.matrix%*%t(pathdev3))
      pathprob3[i,]=c(C[1], C[1]-1.96*std.error, C[1]+1.96*std.error) #prob for path 1,1,0
      
      path2=C[1]-D[1]
      der2=matrix(C[-1]-D[-1], nrow=1)
      std.error2=sqrt(der2%*%V.matrix%*%t(der2))
      results2[i,]=c(path2, path2-1.96*std.error2,path2+1.96*std.error2) #s-> g-> y
      RRder2=matrix(C[-1]/C[1]-D[-1]/D[1], nrow=1)
      RR.std.error2=sqrt(RRder2%*%V.matrix%*%t(RRder2))
      RR2[i,]=c(C[1]/D[1], exp(log(C[1]/D[1])-1.96*RR.std.error2) , exp(log(C[1]/D[1])+1.96*RR.std.error2))
      
      E=point.var.est.cont(1,1,1, beta.hat, alpha.hat, delta.hat,  g.reg, m.reg,confounders) #first part of difference
      G=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg,confounders)
      
      pathdev4=matrix(E[-1], nrow=1)
      std.error=sqrt(pathdev4%*%V.matrix%*%t(pathdev4))
      pathprob4[i,]=c(E[1], E[1]-1.96*std.error, E[1]+1.96*std.error) #prob for path 1,1,1
      
      path3=E[1]-G[1]
      der3=matrix(E[-1]-G[-1], nrow=1)
      std.error3=sqrt(der3%*%V.matrix%*%t(der3))
      results3[i,]=c(path3, path3-1.96*std.error3,path3+1.96*std.error3) #s-> my
      RRder3=matrix(E[-1]/E[1]-G[-1]/G[1], nrow=1)
      RR.std.error3=sqrt(RRder3%*%V.matrix%*%t(RRder3))
      RR3[i,]=c(E[1]/G[1], exp(log(E[1]/G[1])-1.96*RR.std.error3) , exp(log(E[1]/G[1])+1.96*RR.std.error3))
    }
    
    sa=1
    sb=0
    sc=0
    sigm=med.continuous[1]
    sigg=med.continuous[2]
    s.star=sqrt((sigg^2)*(beta.g^2) + 1 + ((beta.m+beta.g*alpha.m)^2)*(sigm^2))
    first=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    sa=0
    second=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    true.rd1=first-second
    true.rr1=first/second
    p1=check(true.rd1,results1) #risk difference number of 95% CI covers
    r1=check(true.rr1,RR1) #risk ratio number of 95% CI covers true
    sa=1
    sb=1
    sc=0
    first=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    sb=0
    second=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    true.rd2=first-second
    true.rr2=first/second
    p2=check(true.rd2,results2)
    r2=check(true.rr2, RR2)
    sa=1
    sb=1
    sc=1
    first=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    sc=0
    second=pnorm((beta.0+beta.s*sa+beta.g*(alpha.0+alpha.s*sb)+(beta.m+beta.g*alpha.m)*(delta.0+delta.s*sc))/s.star)
    true.rd3=first-second
    true.rr3=first/second
    p3=check(true.rd3,results3)
    r3=check(true.rr3, RR3)
    
    p.prob1=mean(pathprob1[,1])
    p.prob2=mean(pathprob2[,1])
    p.prob3=mean(pathprob3[,1])
    p.prob4=mean(pathprob4[,1])
    
    theoretical.rd=sum(true.rd1, true.rd2, true.rd3)
    theoretical.rr=prod(true.rr1, true.rr2, true.rr3)
    # png(filename=paste("bootstrap_", prob.exp, "RiskDifference.png", sep=""))
    # boxplot(cbind(results1[,1]-p1, results2[,1]-p2, results3[,1]-p3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    # png(filename=paste("bootstrap_", prob.exp, "RiskRatio.png", sep=""))
    # boxplot(cbind(RR1[,1]-r1, RR2[,1]-r2, RR3[,1]-r3), xlab="path", names=c("1", "2", "3"), ylab="bias")
    # dev.off()
    
    if (dist.coeff==1){
      correct.coeff=rowSums(confidence.interval.calc)/n.sims
      out.put=c(true.rd1, true.rd2, true.rd3,true.rr1, true.rr2, true.rr3,p1/n.sims,p2/n.sims,p3/n.sims,r1/n.sims,r2/n.sims,r3/n.sims, p.prob1, p.prob2, p.prob3, p.prob4, correct.coeff, total.rd, theoretical.rd, total.rr, theoretical.rr,mean(results1[,1])-true.rd1, sd(results1[,1]), sum((results1[,1]-true.rd1)^2)/n.sims,mean(results2[,1])-true.rd2, sd(results2[,1]), sum((results2[,1]-true.rd2)^2)/n.sims,mean(results3[,1])-true.rd3,sd(results3[,1]), sum((results3[,1]-true.rd3)^2)/n.sims,mean(RR1[,1])-true.rr1, sd(RR1[,1]), sum((RR1[,1]-true.rr1)^2)/n.sims, mean(RR2[,1])-true.rr2, sd(RR2[,1]), sum((RR2[,1]-true.rr2)^2)/n.sims,mean(RR3[,1])-true.rr3, sd(RR3[,1]), sum((RR3[,1]-true.rr3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDdelta path 1", "RDdelta path 2", "RDdelta path 3","RRdelta path 1", "RRdelta path 2", "RRdelta path 3", "path000", "path100", "path110", "path111","beta.0", "beta.s", "beta.m", "beta.g", "alpha.0", "alpha.s", "alpha.m","delta.0", "delta.s", "total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3" )
      return(out.put)#use this if want to know how well the coefficients were simulated
    }
    else{
      out.put=c(true.rd1, true.rd2, true.rd3,true.rr1, true.rr2, true.rr3,p1/n.sims,p2/n.sims,p3/n.sims,r1/n.sims,r2/n.sims,r3/n.sims,p.prob1, p.prob2, p.prob3, p.prob4, total.rd, theoretical.rd, total.rr, theoretical.rr,mean(results1[,1])-true.rd1, sd(results1[,1]), sum((results1[,1]-true.rd1)^2)/n.sims,mean(results2[,1])-true.rd2, sd(results2[,1]), sum((results2[,1]-true.rd2)^2)/n.sims,mean(results3[,1])-true.rd3,sd(results3[,1]), sum((results3[,1]-true.rd3)^2)/n.sims,mean(RR1[,1])-true.rr1, sd(RR1[,1]), sum((RR1[,1]-true.rr1)^2)/n.sims, mean(RR2[,1])-true.rr2, sd(RR2[,1]), sum((RR2[,1]-true.rr2)^2)/n.sims,mean(RR3[,1])-true.rr3, sd(RR3[,1]), sum((RR3[,1]-true.rr3)^2)/n.sims)
      names(out.put)=c("true RDpath1 S->Y", "true RDpath2 S->G->Y", "true RDpath3 S->MY","true RRpath1 S->Y", "true RRpath2 S->G->Y", "true RRpath3 S->MY", "RDdelta path 1", "RDdelta path 2", "RDdelta path 3","RRdelta path 1", "RRdelta path 2", "RRdelta path 3", "path000", "path100", "path110", "path111","total effect RD","theoretical total RD", "total effect RR", "theoretical total RR","RD bias path 1", "RD sd path 1",  "RD MSE path 1" ,"RD bias path 2","RD sd path 2",  "RD MSE path 2" ,"RD bias path 3","RD sd path 3",  "RD MSE path 3" ,"RR bias path 1","RR sd path 1",  "RR MSE path 1" ,"RR bias path 2","RR sd path 2",  "RR MSE path 2" ,"RR bias path 3","RR sd path 3",  "RR MSE path 3" )
      return(out.put)
    }
  }
}

mediation.analysis=function(med.continuous, data.set, confounders){
  data.set=as.data.frame(data.set)
  names(data.set)[c(1:4)]=c("Y", "S", "M", "G")
  #data.set has to be in format: Y, S, M, G, X (any confounders)
  if(med.continuous==0){ #means dichotomous mediators
    outcome.reg=glm(Y~., family = binomial (link="logit"), data=data.set)
    g.reg=glm(G~.-Y, family = binomial (link="logit"), data=data.set)
    m.reg=glm(M~.-Y-G, family = binomial (link="logit"), data=data.set)
    beta.hat=summary(outcome.reg)$coefficients[,1]
    alpha.hat=summary(g.reg)$coefficients[,1]
    delta.hat=summary(m.reg)$coefficients[,1]
    
    V.matrix=create.vmatrix(outcome.reg, g.reg, m.reg)
    total.effect=summary(glm(Y~.-G-M, family = binomial (link="logit"), data=data.set))$coefficients[,1]
    total.effect.1=exp(sum(total.effect*c(1,1,confounders)))/(1+exp(sum(total.effect*c(1,1,confounders))))
    total.effect.2=exp(sum(total.effect*c(1,0,confounders)))/(1+exp(sum(total.effect*c(1,0,confounders))))
    total.rd=total.effect.1-total.effect.2 #when S=1 compared to S=0
    total.rr=total.effect.1/total.effect.2
    
    A=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,confounders) #first part of difference
    B=point.var.est.di(0,0,0, beta.hat, alpha.hat, delta.hat,confounders)
    C=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,confounders) #first part of difference
    D=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,confounders)
    E=point.var.est.di(1,1,1, beta.hat, alpha.hat, delta.hat,confounders) #first part of difference
    G=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,confounders)
  }
  else{
    #do the continuous
    outcome.reg=glm(Y~., family = binomial (link="probit"), data=data.set)
    g.reg=lm(G~.-Y, data=data.set)
    m.reg=lm(M~.-G-Y, data=data.set)
    beta.hat=summary(outcome.reg)$coefficients[,1]
    alpha.hat=summary(g.reg)$coefficients[,1]
    delta.hat=summary(m.reg)$coefficients[,1]
    V.matrix=create.vmatrix(outcome.reg, g.reg, m.reg)
    total.effect=summary(glm(Y~.-M-G, family = binomial (link="probit"), data=data.set))$coefficients[,1]
    total.rd=pnorm(sum(total.effect*c(1,1,confounders)))-pnorm(sum(total.effect*c(1,0,confounders))) #when S=1 compared to S=0
    total.rr=pnorm(sum(total.effect*c(1,1,confounders)))/pnorm(sum(total.effect*c(1,0,confounders))) 
    
    A=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg, confounders) #first part of difference
    B=point.var.est.cont(0,0,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg,confounders)
    C=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg,confounders) #first part of difference
    D=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat,g.reg, m.reg,confounders)
    E=point.var.est.cont(1,1,1, beta.hat, alpha.hat, delta.hat,g.reg, m.reg, confounders) #first part of difference
    G=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg, confounders)
  }
  pathprob1=B[1] #prob for path 0,0,0
  pathprob2=A[1]#prob for path 1,0,0
  pathprob3=C[1] #prob for path 1,1,0
  pathprob4=E[1] #prob for path 1,1,1
  
  #path 1, p-values were calculated from a normal z-distribution. can replace pnorm with pt function for t-distribution, but would need to specify sample size.
  path=A[1]-B[1] #point estimate of probability of this path
  der=matrix(A[-1]-B[-1], nrow=1)
  std.error=sqrt(der%*%V.matrix%*%t(der))
  results1=c(path, path-1.96*std.error,path+1.96*std.error) #s->y
  pval.rd1=2*min(pnorm(abs(path/std.error)),1-pnorm(abs(path/std.error)))
  path.rr=A[1]/B[1]
  RRder1=matrix((A[-1]/A[1])-(B[-1]/B[1]),nrow=1)
  RR.std.error=sqrt(RRder1%*%V.matrix%*%t(RRder1))
  RR1=c(path.rr, exp(log(path.rr)-1.96*RR.std.error) , exp(log(path.rr)+1.96*RR.std.error))
  pval.rr1=2*min(pnorm(abs(log(path.rr)/RR.std.error)),1-pnorm(abs(log(path.rr)/RR.std.error)))
  
  #path 2
  path2=C[1]-D[1]
  der2=matrix(C[-1]-D[-1], nrow=1)
  std.error2=sqrt(der2%*%V.matrix%*%t(der2))
  results2=c(path2, path2-1.96*std.error2,path2+1.96*std.error2) #s-> g-> y
  pval.rd2=2*min(pnorm(abs(path2/std.error2)),1-pnorm(abs(path2/std.error2)))
  RRder2=matrix(C[-1]/C[1]-D[-1]/D[1], nrow=1)
  RR.std.error2=sqrt(RRder2%*%V.matrix%*%t(RRder2))
  RR2=c(C[1]/D[1], exp(log(C[1]/D[1])-1.96*RR.std.error2) , exp(log(C[1]/D[1])+1.96*RR.std.error2))
  pval.rr2=2*min(pnorm(abs(log(C[1]/D[1])/RR.std.error2)),1-pnorm(abs(log(C[1]/D[1])/RR.std.error2)))
  
  #path 3
  path3=E[1]-G[1]
  der3=matrix(E[-1]-G[-1], nrow=1)
  std.error3=sqrt(der3%*%V.matrix%*%t(der3))
  results3=c(path3, path3-1.96*std.error3,path3+1.96*std.error3) #s-> my
  pval.rd3=2*min(pnorm(abs(path3/std.error3)),1-pnorm(abs(path3/std.error3)))
  RRder3=matrix(E[-1]/E[1]-G[-1]/G[1], nrow=1)
  RR.std.error3=sqrt(RRder3%*%V.matrix%*%t(RRder3))
  RR3=c(E[1]/G[1], exp(log(E[1]/G[1])-1.96*RR.std.error3) , exp(log(E[1]/G[1])+1.96*RR.std.error3))
  pval.rr3=2*min(pnorm(abs(log(E[1]/G[1])/RR.std.error3)),1-pnorm(abs(log(E[1]/G[1])/RR.std.error3)))
  
  theoretical.rd=sum(results1[1], results2[1], results3[1])
  theoretical.rr=prod(RR1[1], RR2[1], RR3[1])
  
  
  
  out.put=c(results1,pval.rd1,results2,pval.rd2,results3,pval.rd3 ,RR1,pval.rr1, RR2,pval.rr2, RR3, pval.rr3,pathprob1,pathprob2,pathprob3,pathprob4, total.rd,theoretical.rd, total.rr,theoretical.rr)
  names(out.put)=c("RDpath1 S->Y", "lower95%", "upper95%","pval1" ,"RDpath2 S->G->Y", "lower95%", "upper95%","pval2", "RDpath3 S->MY", "lower95%", "upper95%","pval3","RRpath1 S->Y", "lower95%", "upper95%","pval1RR","RRpath2 S->G->Y", "lower95%", "upper95%","pval2RR","RRpath3 S->MY", "lower95%", "upper95%","pvalRR3","prob000","prob100","prob110","prob111","total effect RD","theoretical total RD", "total effect RR", "theoretical total RR")
  return(out.put)
}

#bootstrap mediation analysis, #n.boot= number of resampling to get 95%CI
boot.mediation.analysis=function(med.continuous, data.set, confounders,n.boot){
  data.set=as.data.frame(data.set)
  names(data.set)[c(1:4)]=c("Y", "S", "M", "G")
  #data.set has to be in format: Y, S, M, G, X (any confounders)
  if(med.continuous==0){ #means dichotomous mediators
    outcome.reg=glm(Y~., family = binomial (link="logit"), data=data.set)
    g.reg=glm(G~.-Y, family = binomial (link="logit"), data=data.set)
    m.reg=glm(M~.-Y-G, family = binomial (link="logit"), data=data.set)
    beta.hat=summary(outcome.reg)$coefficients[,1]
    alpha.hat=summary(g.reg)$coefficients[,1]
    delta.hat=summary(m.reg)$coefficients[,1]
    
    total.effect=summary(glm(Y~.-G-M, family = binomial (link="logit"), data=data.set))$coefficients[,1]
    total.effect.1=exp(sum(total.effect*c(1,1,confounders)))/(1+exp(sum(total.effect*c(1,1,confounders))))
    total.effect.2=exp(sum(total.effect*c(1,0,confounders)))/(1+exp(sum(total.effect*c(1,0,confounders))))
    total.rd=total.effect.1-total.effect.2 #when S=1 compared to S=0
    total.rr=total.effect.1/total.effect.2
    
    A=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,confounders) #first part of difference
    B=point.var.est.di(0,0,0, beta.hat, alpha.hat, delta.hat,confounders)
    C=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,confounders) #first part of difference
    D=point.var.est.di(1,0,0, beta.hat, alpha.hat, delta.hat,confounders)
    E=point.var.est.di(1,1,1, beta.hat, alpha.hat, delta.hat,confounders) #first part of difference
    G=point.var.est.di(1,1,0, beta.hat, alpha.hat, delta.hat,confounders)

    #bootstrap
    var.boot=create.var.boot.di(n.boot, data.set, confounders) 
    
  }
  else{
    #do the continuous stuff
    outcome.reg=glm(Y~., family = binomial (link="probit"), data=data.set)
    g.reg=lm(G~.-Y, data=data.set)
    m.reg=lm(M~.-G-Y, data=data.set)
    beta.hat=summary(outcome.reg)$coefficients[,1]
    alpha.hat=summary(g.reg)$coefficients[,1]
    delta.hat=summary(m.reg)$coefficients[,1]
    total.effect=summary(glm(Y~.-M-G, family = binomial (link="probit"), data=data.set))$coefficients[,1]
    total.rd=pnorm(sum(total.effect*c(1,1,confounders)))-pnorm(sum(total.effect*c(1,0,confounders))) #when S=1 compared to S=0
    total.rr=pnorm(sum(total.effect*c(1,1,confounders)))/pnorm(sum(total.effect*c(1,0,confounders))) 
    
    A=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg,confounders) #first part of difference
    B=point.var.est.cont(0,0,0, beta.hat, alpha.hat, delta.hat,g.reg, m.reg, confounders)
    C=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat, g.reg, m.reg, confounders) #first part of difference
    D=point.var.est.cont(1,0,0, beta.hat, alpha.hat, delta.hat,g.reg, m.reg, confounders)
    E=point.var.est.cont(1,1,1, beta.hat, alpha.hat, delta.hat, g.reg, m.reg, confounders) #first part of difference
    G=point.var.est.cont(1,1,0, beta.hat, alpha.hat, delta.hat,g.reg, m.reg, confounders)
    
    var.boot=create.var.boot.cont(n.boot, data.set, confounders)
  }
  pathprob1=B[1] #prob for path 0,0,0
  pathprob2=A[1]#prob for path 1,0,0
  pathprob3=C[1] #prob for path 1,1,0
  pathprob4=E[1] #prob for path 1,1,1
  
  #path 1
  path=A[1]-B[1] #point estimate of probability of this path
  results1=c(path) #s->y
  path.rr=A[1]/B[1]
  
  #path 2
  path2=C[1]-D[1]
  path.rr2=C[1]/D[1]
  
  #path 3
  path3=E[1]-G[1]
  path.rr3=E[1]/G[1]
  theoretical.rd=sum(path, path2, path3)
  theoretical.rr=prod(path.rr, path.rr2, path.rr3)
  
  b.RD1=c(quantile(var.boot[,1], probs=c(0.025, 0.975)))
  b.RD2=c(quantile(var.boot[,2], probs=c(0.025, 0.975)))
  b.RD3=c(quantile(var.boot[,3], probs=c(0.025, 0.975)))
  b.RR1=c(quantile(var.boot[,4], probs=c(0.025, 0.975)))
  b.RR2=c(quantile(var.boot[,5], probs=c(0.025, 0.975)))
  b.RR3=c(quantile(var.boot[,6], probs=c(0.025, 0.975)))
  
  total=dim(var.boot)[1]
#null hypothesis is 0 no effect in path
  valRD1=sum(var.boot[,1]>0)/total
  pvalRD1=min(valRD1,(1-valRD1))*2
  valRD2=sum(var.boot[,2]>0)/total
  pvalRD2=min(valRD2,(1-valRD2))*2
  valRD3=sum(var.boot[,3]>0)/total
  pvalRD3=min(valRD3,(1-valRD3))*2
  
  #null hypothesis is log(1) no difference in log(relative risk). This is done in the log scale since it is more normally distributed
  valRR1=sum(log(var.boot[,4])>0)/total
  pvalRR1=min(valRR1,(1-valRR1))*2
  valRR2=sum(log(var.boot[,5])>0)/total
  pvalRR2=min(valRR2,(1-valRR2))*2
  valRR3=sum(log(var.boot[,6])>0)/total
  pvalRR3=min(valRR3,(1-valRR3))*2
  
  out.put=c(path,b.RD1, pvalRD1,path2,b.RD2, pvalRD2, path3, b.RD3,pvalRD3, path.rr,b.RR1, pvalRR1, path.rr2,b.RR2,pvalRR2,path.rr3,b.RR3,pvalRR3, pathprob1,pathprob2,pathprob3,pathprob4, total.rd,theoretical.rd, total.rr,theoretical.rr)
    names(out.put)=c("RDpath1 S->Y", "bootlower95%", "bootupper95%","pval1", "RDpath2 S->G->Y", "bootlower95%", "bootupper95%","pval2","RDpath3 S->MY", "bootlower95%", "bootupper95%","pval3","RRpath1 S->Y", "bootlower95%", "bootupper95%","RRpval1","RRpath2 S->G->Y", "bootlower95%", "bootupper95%","RRpval2","RRpath3 S->MY", "bootlower95%", "bootupper95%","RRpval3","prob000","prob100","prob110","prob111","total effect RD","theoretical total RD", "total effect RR", "theoretical total RR")
    return(out.put)
  }

#Edits also need to be made to the following functions if mediators are dichotomous
dev.comp=function(G,M,b0,bs,bm,bg,bX, a0, as, am, aX, d0,ds,dX, confounders, sa,sb,sc ){
  D=exp(b0+bs*sa+bm*M+bg*G + sum(bX*confounders)) #always the same equations
  E=exp(a0+as*sb+am*M + sum(aX*confounders))
  H=exp(d0+ds*sc+ sum(dX*confounders))
  if (G==1){
    if (M==1){
      dd0= (D/(1+D))*(E/(1+E))*(H/((1+H)^2)) #derivative for delta 0
      da0=(D/(1+D))*(H/(1+H))*(E/((1+E)^2)) #derivative for alpha 0
      dB0= (E/(1+E))*(H/(1+H))*(D/((1+D)^2)) #derivative beta 0
      
      p=D*E*H/((1+D)*(1+E)*(1+H))
    }
    if (M==0){
      dd0= -(D/(1+D))*(E/(1+E))*(H/(1+H)^(2)) #derivative for delta 0
      da0=(D/(1+D))*(1-(H/(1+H)))*(E/((1+E)^2)) #derivative for alpha 0
      dB0= (E/(1+E))*(1-(H/(1+H)))*(D/((1+D)^2)) #derivative beta 0
      p=(D*E/((1+D)*(1+E)))*(1-(H/(1+H)))
    }
  }
  if (G==0){
    if (M==1){
      dd0= (D/(1+D))*(1-(E/(1+E)))*(H/((1+H)^2)) #derivative for delta 0
      da0=-(D/(1+D))*(H/(1+H))*(E/((1+E)^2)) #derivative for alpha 0
      dB0= (H/(1+H))*(1-(E/(1+E)))*(D/((1+D)^2)) #derivative beta 0
      p=(D*H/((1+D)*(1+H)))*(1-(E/(1+E)))
    }
    if (M==0){
      dd0= -(D/(1+D))*(1-(E/(1+E)))*(H/(1+H)^(2)) #derivative for delta 0
      da0= -(D/(1+D))*(1-(H/(1+H)))*(E/((1+E)^2)) #derivative for alpha 0
      dB0= (1-(E/(1+E)))*(1-(H/(1+H)))*(D/((1+D)^2)) #derivative beta 0
      p=(D/(1+D))*(1-(E/(1+E)))*(1-(H/(1+H)))
    }
  }
  dds= dd0*sc #derivative for delta s
  das= da0*sb #derivative for alpha s
  dam= da0*M #derivative for alpha m
  dBs= dB0*sa #derivative beta s
  dBm= dB0*M #derivative beta m 
  dBg= dB0*G #derivative beta g 
  dBX= dB0*confounders
  daX= da0*confounders
  ddX= dd0*confounders
  if (length(confounders)==0){
    return(c(p,dB0,dBs, dBm, dBg, da0,das,dam,dd0,dds))
  }
  else{return(c(p,dB0,dBs, dBm, dBg, dBX, da0,das,dam,daX, dd0,dds,ddX))
  }
}

point.var.est.di=function(sa,sb,sc,beta.hat, alpha.hat, delta.hat, confounders){
  b0=beta.hat[1]
  bs=beta.hat[2]
  bm=beta.hat[3]
  bg=beta.hat[4]
  beta.rest=beta.hat[-(1:4)]
  a0=alpha.hat[1]
  as=alpha.hat[2]
  am=alpha.hat[3]
  alpha.rest=alpha.hat[-(1:3)]
  d0=delta.hat[1]
  ds=delta.hat[2]
  delta.rest=delta.hat[-(1:2)]
  
  
  
  #to calc derivatives 
  dev1=dev.comp(G=1,M=1,b0,bs,bm,bg,beta.rest,a0, as, am, alpha.rest,d0,ds,delta.rest ,confounders, sa,sb,sc)
  dev2=dev.comp(G=1,M=0,b0,bs,bm,bg,beta.rest,a0, as, am, alpha.rest,d0,ds,delta.rest ,confounders,sa,sb,sc)
  dev3=dev.comp(G=0,M=1,b0,bs,bm,bg,beta.rest,a0, as, am, alpha.rest,d0,ds,delta.rest ,confounders,sa,sb,sc)
  dev4=dev.comp(G=0,M=0,b0,bs,bm,bg,beta.rest,a0, as, am, alpha.rest,d0,ds,delta.rest ,confounders,sa,sb,sc)
  
  #combine all the dervatives into a vector, order based on the covariance matrices
  derivatives=matrix(dev1+dev2+dev3+dev4, nr=1)
  return(derivatives)
}

create.var.boot.di=function(n.boot, sim.data1, confounders){
  var.boot1=matrix(NA, nr=n.boot, nc=6) #store all the path effects path 1, path 2, path3
  s.size=dim(sim.data1)[1]
  for (boot.count in 1:n.boot){
    set.seed(37+boot.count*3)
    rv.normal.boot=as.data.frame(sim.data1[sample(1:s.size, replace=TRUE),]) #sampling same size as the data set
    b.outcome.reg=glm(Y~., family = binomial (link="logit"), data=rv.normal.boot)
    b.g.reg=glm(G~.-Y, family = binomial (link="logit"), data=rv.normal.boot)
    b.m.reg=glm(M~.-Y-G, family = binomial (link="logit"), data=rv.normal.boot)
    b.beta.hat=summary(b.outcome.reg)$coefficients[,1]
    b.alpha.hat=summary(b.g.reg)$coefficients[,1]
    b.delta.hat=summary(b.m.reg)$coefficients[,1]
    
    b.A=point.var.est.di(1,0,0, b.beta.hat, b.alpha.hat, b.delta.hat,confounders) #first part of difference
    b.B=point.var.est.di(0,0,0, b.beta.hat, b.alpha.hat, b.delta.hat,confounders)
    b.path=b.A[1]-b.B[1] #point estimate of probability of this path
    b.path1.r=b.A[1]/b.B[1]
    b.C=point.var.est.di(1,1,0, b.beta.hat, b.alpha.hat, b.delta.hat,confounders) #first part of difference
    b.D=point.var.est.di(1,0,0, b.beta.hat, b.alpha.hat, b.delta.hat,confounders) 
    b.path2=b.C[1]-b.D[1]
    b.path2.r=b.C[1]/b.D[1]
    b.E=point.var.est.di(1,1,1, b.beta.hat, b.alpha.hat, b.delta.hat,confounders)  #first part of difference
    b.G=point.var.est.di(1,1,0, b.beta.hat, b.alpha.hat, b.delta.hat,confounders)
    b.path3=b.E[1]-b.G[1]
    b.path3.r=b.E[1]/b.G[1]
    var.boot1[boot.count,]=c(b.path, b.path2, b.path3, b.path1.r, b.path2.r, b.path3.r)
  }
  return(var.boot1)
}

#Edits also need to be made to the following function if mediators are continuous
point.var.est.cont=function(sa,sb,sc,beta.hat, alpha.hat, delta.hat, g.reg,m.reg,confounders){
  b0=beta.hat[1]
  bs=beta.hat[2]
  bm=beta.hat[3]
  bg=beta.hat[4]
  beta.rest=beta.hat[-(1:4)]
  bX=beta.rest
  a0=alpha.hat[1]
  as=alpha.hat[2]
  am=alpha.hat[3]
  alpha.rest=alpha.hat[-(1:3)]
  aX=alpha.rest
  d0=delta.hat[1]
  ds=delta.hat[2]
  delta.rest=delta.hat[-(1:2)]
  dX=delta.rest
  #calc sigmas
  sigma.hat.g=sqrt(mean((g.reg$residuals)^2)) #Mean squared error, double check
  sigma.hat.m=sqrt(mean((m.reg$residuals)^2))
  sigma.star= sqrt((sigma.hat.g^2)*(bg^2) + 1 + ((bm+bg*am)^2)*(sigma.hat.m^2))  
  
  #############
  #the following needs to be changed to reflect potential interactions
  p=(b0+sum(bX*confounders)+bs*sa+bg*(a0+as*sb+sum(aX*confounders))+(bm+bg*am)*(d0+ds*sc+sum(dX*confounders)))/sigma.star
  g.x=(b0+sum(bX*confounders)+bs*sa+bg*(a0+as*sb+sum(aX*confounders))+(bm+bg*am)*(d0+ds*sc+sum(dX*confounders)))
  
  dB0= (exp(-(g.x^2)/(2*(sigma.star^2))))/(sqrt(2*pi)*sigma.star) #derivative beta 0
  dBs=dB0* sa #derivative beta s
  dBm= dB0*(sigma.star*(d0+ds*sc+sum(dX*confounders))-((g.x/sigma.star)*(bm+bg*am)*sigma.hat.m^2))/(sigma.star) #derivative beta m 
  dBg= dB0*(sigma.star*(a0+sum(aX*confounders)+as*sb+am*(d0+ds*sc+sum(dX*confounders)))-(g.x/sigma.star)*(bg*sigma.hat.g^2+(bg*am+bm)*am*sigma.hat.m^2))/sigma.star #derivative beta g 
  da0=dB0*bg#derivative for alpha 0
  das=da0*sb#derivative for alpha s
  dam=da0*(sigma.star*(d0+ds*sc+sum(dX*confounders))-(g.x/sigma.star)*(bm+bg*(am))*(sigma.hat.m^2))/sigma.star #derivative for alpha m
  
  dd0=dB0*(bm+bg*am)#derivative for delta 0
  dds=dd0*sc#derivative for delta s
  
  dBX=dB0*confounders
  daX=da0*confounders
  ddX=dd0*confounders
  ######
  
  #combine all the dervatives into a vector, order based on the covariance matrices
  if (length(confounders)==0){
    derivatives=c( dB0, dBs, dBm, dBg,da0, das, dam, dd0, dds)
    return(c(pnorm(p), as.vector(derivatives)))
  }
  else{
    derivatives=c( dB0, dBs, dBm, dBg,dBX, da0, das, dam, daX, dd0, dds,ddX )
    return(c(pnorm(p), as.vector(derivatives)))
  }
  
}

create.var.boot.cont=function(n.boot, sim.data1, confounders){
  var.boot1=matrix(NA, nr=n.boot, nc=6)
  s.size=dim(sim.data1)[1]
  for (boot.count in 1:n.boot){
    set.seed(37+boot.count*3)
    rv.normal.boot=as.data.frame(sim.data1[sample(1:s.size, replace=TRUE),]) #sampling same size as the data set
    b.outcome.reg=glm(Y~., family = binomial (link="probit"), data=rv.normal.boot)
    b.g.reg=lm(G~.-Y, data=rv.normal.boot)
    b.m.reg=lm(M~.-Y-G,  data=rv.normal.boot)
    b.beta.hat=summary(b.outcome.reg)$coefficients[,1]
    b.alpha.hat=summary(b.g.reg)$coefficients[,1]
    b.delta.hat=summary(b.m.reg)$coefficients[,1]
    
    b.A=point.var.est.cont(1,0,0, b.beta.hat, b.alpha.hat, b.delta.hat, b.g.reg, b.m.reg,confounders) #first part of difference
    b.B=point.var.est.cont(0,0,0, b.beta.hat, b.alpha.hat, b.delta.hat, b.g.reg, b.m.reg,confounders)
    b.path=b.A[1]-b.B[1] #point estimate of probability of this path
    b.path.r=b.A[1]/b.B[1]
    b.C=point.var.est.cont(1,1,0, b.beta.hat, b.alpha.hat, b.delta.hat, b.g.reg, b.m.reg,confounders) #first part of difference
    b.D=point.var.est.cont(1,0,0, b.beta.hat, b.alpha.hat, b.delta.hat, b.g.reg, b.m.reg,confounders)
    b.path2=b.C[1]-b.D[1]
    b.path2.r=b.C[1]/b.D[1]
    b.E=point.var.est.cont(1,1,1, b.beta.hat, b.alpha.hat, b.delta.hat, b.g.reg, b.m.reg,confounders) #first part of difference
    b.G=point.var.est.cont(1,1,0, b.beta.hat, b.alpha.hat, b.delta.hat, b.g.reg, b.m.reg,confounders)
    b.path3=b.E[1]-b.G[1]
    b.path3.r=b.E[1]/b.G[1]
    var.boot1[boot.count,]=c(b.path, b.path2, b.path3, b.path.r, b.path2.r, b.path3.r)
  }
  return(var.boot1)
}

#the following functions do not need to be changed
check=function(value, result.matrix){ 
  sum(result.matrix[,2]<value & result.matrix[,3]>value)
}

create.vmatrix=function(o.reg, g.reg1, m.reg1){
  beta.hat1=length(summary(o.reg)$coefficients[,1])
  alpha.hat1=length(summary(g.reg1)$coefficients[,1])
  delta.hat1=length(summary(m.reg1)$coefficients[,1])
  
  dimensions=sum(beta.hat1+alpha.hat1+delta.hat1)
  V.matrix.1=matrix(0, nr=dimensions, nc=dimensions)
  V.matrix.1[c(1:beta.hat1),c(1:beta.hat1)]=vcov(o.reg)
  V.matrix.1[c((1+beta.hat1):(beta.hat1+alpha.hat1)),c((1+beta.hat1):(beta.hat1+alpha.hat1))]=vcov(g.reg1)
  V.matrix.1[c((1+beta.hat1+alpha.hat1):(beta.hat1+alpha.hat1+delta.hat1)),c((1+beta.hat1+alpha.hat1):(beta.hat1+alpha.hat1+delta.hat1))]=vcov(m.reg1)
  return (V.matrix.1)
}


#optional function to assess simulated data----
library ("plyr")
test.sim.setting=function(n.sims, med.continuous,exp.continuous,sample.size,prob.exp, beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s){
  out.put.freq=matrix(NA, nc=6, nr=n.sims)
    for (j in 1:n.sims){
      d=as.data.frame(sim.mediation.data(med.continuous,exp.continuous,sample.size,prob.exp, j,  beta.0, beta.s, beta.m, beta.g, alpha.0, alpha.s, alpha.m, delta.0, delta.s))
      names(d)=c("Y", "S","M", "G")
      out.put.freq[j,]=cbind(sum(d$Y), length(count(d)$freq), sum(d$Y==0), min(count(d)$freq), sum(d$M), sum(d$G)) #check it adds to sample size
      #length(count(d)$freq) should be 16 for dichotomous
    }
    return(as.data.frame(out.put.freq))
  }


#code to assess one mediator problems----
#Code to calculate natural direct andindirect effects based on Linda Valeri and Tyler VanderWeele paper 2013 in Psychological Methods and Vanderweele 2010 supplement page 2, and no interaction terms were considered


#for 1 binary mediator and binary outcome:----
binarymediator=function(data.input,confounders,n.boot){
  outcome.reg=glm(Y~., family=binomial(link="logit"), data=data.input)
  #equation from paper: logit(P(Y=1|A=a, M=m, C=c))=theta0+theta1A+theta2m+theta3C
  
  mediator.reg=glm(M~.-Y, family=binomial(link="logit"), data=data.input)
  #equation from paper: logit(P(M=1|A=a,C=c))=beta0+beta1A+beta2C
  
  #natural direct effect
  NDE=exp(outcome.reg$coefficients[2])
  
  #natural indirect effect
  mediator.confoundercoeff=mediator.reg$coefficients[c(3:(dim(data.input)[2]-1))]
  #exp(beta0 + beta2*confounders)
  A=exp(mediator.reg$coefficients[1]+sum(mediator.confoundercoeff*confounders))
  #exp(theta2+beta0+beta1+beta2*confounders)
  B=exp(outcome.reg$coefficients[3]+mediator.reg$coefficients[1]+mediator.reg$coefficients[2]+sum(mediator.confoundercoeff*confounders))
  #exp(beta0 + beta1+beta2*confounders)
  C=exp(mediator.reg$coefficients[1]+mediator.reg$coefficients[2]+sum(mediator.confoundercoeff*confounders))
  #exp(theta2+beta0+beta2*confounders)
  D=exp(outcome.reg$coefficients[3]+mediator.reg$coefficients[1]+sum(mediator.confoundercoeff*confounders))
  
  NIE=((1+A)*(1+B))/((1+C)*(1+D))
  
  #delta method: mediator coefficient values followed by outcome coefficient values
  dimensions=sum(length(outcome.reg$coefficients)+length(mediator.reg$coefficients))
  V.matrix=matrix(0, nr=dimensions, nc=dimensions)
  V.matrix[c(1:length(mediator.reg$coefficients)), c(1:length(mediator.reg$coefficients))]=vcov(mediator.reg)
  V.matrix[c((1+length(mediator.reg$coefficients)):(length(mediator.reg$coefficients)+length(outcome.reg$coefficients))), c((1+length(mediator.reg$coefficients)):(length(mediator.reg$coefficients)+length(outcome.reg$coefficients)))]=vcov(outcome.reg)
  V.matrix[c((1+length(mediator.reg$coefficients)):(length(mediator.reg$coefficients)+length(outcome.reg$coefficients))),c((1+length(mediator.reg$coefficients)):(length(mediator.reg$coefficients)+length(outcome.reg$coefficients)))]=vcov(outcome.reg)
  
  NDE.derivatives=c(0,0,0*confounders,0,1,0,0*confounders)
  NDE.var=sqrt(NDE.derivatives%*%V.matrix%*%NDE.derivatives)
  NDE.CI=c(NDE, exp(log(NDE)-1.96*NDE.var) , exp(log(NDE)+1.96*NDE.var))
  
  d.b0=((A*(1+B)+(1+A)*B)/((1+C)*(1+D)))-((1+A)*(1+B)*((C/((1+D)*(1+C)^2))+(D/((1+C)*(1+D)^2))))
  d.b1=((1+A)*B/((1+C)*(1+D)))-((1+A)*(1+B)*((C/((1+D)*(1+C)^2))))
  d.theta2=(1+A)*(B-D)/((1+C)*(1+D)^2)
  
  NIE.derivatives=c(d.b0,d.b1,d.b0*confounders,0,0,d.theta2,0*confounders)
  NIE.var=sqrt(NIE.derivatives%*%V.matrix%*%NIE.derivatives)
  NIE.CI=c(NIE, exp(log(NIE)-1.96*NIE.var) , exp(log(NIE)+1.96*NIE.var))
  
  #bootstrap
  var.boot=matrix(NA, nr=n.boot, nc=2) #record NDE and NIE for each resampling 
  s.size=dim(data.input)[1]
  for (boot.count in 1:n.boot){
    set.seed(37+boot.count*3)
    rv.normal.boot=as.data.frame(data.input[sample(1:s.size, replace=TRUE),]) 
    outcome.reg=glm(Y~., family = binomial (link="logit"),data=rv.normal.boot)
    mediator.reg=glm(M~.-Y,family = binomial (link="logit"), data=rv.normal.boot)
    NDE=exp(outcome.reg$coefficients[2])
    #natural indirect effect
    mediator.confoundercoeff=mediator.reg$coefficients[c(3:(dim(data.input)[2]-1))]
    #exp(beta0 + beta2*confounders)
    A=exp(mediator.reg$coefficients[1]+sum(mediator.confoundercoeff*confounders))
    #exp(theta2 + beta0 + beta1+ beta2*confounders)
    B=exp(outcome.reg$coefficients[3]+mediator.reg$coefficients[1]+mediator.reg$coefficients[2]+sum(mediator.confoundercoeff*confounders))
    #exp(beta0 + beta1+beta2*confounders)
    C=exp(mediator.reg$coefficients[1]+mediator.reg$coefficients[2]+sum(mediator.confoundercoeff*confounders))
    #exp(theta2 + beta0 + beta1+ beta2*confounders)
    D=exp(outcome.reg$coefficients[3]+mediator.reg$coefficients[1]+sum(mediator.confoundercoeff*confounders))
    
    NIE=((1+A)*(1+B))/((1+C)*(1+D))
    
    var.boot[boot.count,]=c(NDE, NIE)
  }
  b.NDE=c(quantile(var.boot[,1], probs=c(0.025, 0.975)))
  b.NIE=c(quantile(var.boot[,2], probs=c(0.025, 0.975)))
  out.put=c(NDE.CI, b.NDE,NIE.CI,  b.NIE)
  names(out.put)=c("NDE", "deltalower95%CI","deltaupper95%CI", "bootlower95%CI","bootupper95%CI","NIE", "deltalower95%CI","deltaupper95%CI","bootlower95%CI","bootupper95%CI")
  return(out.put)
  
}

#for 1 continuous mediator and binary outcome---- 
continuousmediator=function(data.input,confounders, n.boot){
  data.input=as.data.frame(data.input)
  names(data.input)[c(1:3)]=c("Y","S", "M")
  outcome.reg=glm(Y~., family = binomial (link="logit"), data=data.input)
  #equation from paper: logit(P(M=1|A=a, C=c))=beta0+beta1A+beta2C
  mediator.reg=lm(M~.-Y, data=data.input)
  #equation from paper: logit(P(Y=1|A=a, M=m, C=c))=theta0+theta1A+theta2m +theta3C
  
  #natural direct effect, exp(theta1)
  NDE=exp(outcome.reg$coefficients[2])
  
  #natural indirect effect, exp(theta2*beta1)
  NIE=exp(outcome.reg$coefficients[3]*mediator.reg$coefficients[2])
  
  #delta method: mediator coefficient values than put outcome coefficent values
  dimensions=sum(length(outcome.reg$coefficients)+length(mediator.reg$coefficients))
  V.matrix=matrix(0, nr=dimensions, nc=dimensions)
  V.matrix[c(1:length(mediator.reg$coefficients)),c(1:length(mediator.reg$coefficients))]=vcov(mediator.reg)
  V.matrix[c((1+length(mediator.reg$coefficients)):(length(mediator.reg$coefficients)+length(outcome.reg$coefficients))),c((1+length(mediator.reg$coefficients)):(length(mediator.reg$coefficients)+length(outcome.reg$coefficients)))]=vcov(outcome.reg)
  
  NDE.derivatives=c(0,0,0*confounders,0,1,0,0*confounders)
  NDE.var=sqrt(NDE.derivatives%*%V.matrix%*%NDE.derivatives)
  NDE.CI=c(NDE, exp(log(NDE)-1.96*NDE.var) , exp(log(NDE)+1.96*NDE.var))
  
  NIE.derivatives=c(0,outcome.reg$coefficients[3],0*confounders,0,0,mediator.reg$coefficients[2],0*confounders)
  NIE.var=sqrt(NIE.derivatives%*%V.matrix%*%NIE.derivatives)
  NIE.CI=c(NIE, exp(log(NIE)-1.96*NIE.var) , exp(log(NIE)+1.96*NIE.var))
  
  #bootstrap
  var.boot=matrix(NA, nr=n.boot, nc=2) #record NDE and NIE for each resampling 
  s.size=dim(data.input)[1]
  for (boot.count in 1:n.boot){
    set.seed(37+boot.count*3)
    rv.normal.boot=as.data.frame(data.input[sample(1:s.size, replace=TRUE),]) 
    outcome.reg=glm(Y~.,family = binomial (link="logit"), data=rv.normal.boot)
    mediator.reg=lm(M~.-Y, data=rv.normal.boot)
    NDE=exp(outcome.reg$coefficients[2])
    NIE=exp(outcome.reg$coefficients[3]*mediator.reg$coefficients[2])
    var.boot[boot.count,]=c(NDE, NIE)
  }
  b.NDE=c(quantile(var.boot[,1], probs=c(0.025, 0.975)))
  b.NIE=c(quantile(var.boot[,2], probs=c(0.025, 0.975)))
  out.put=c(NDE.CI, b.NDE,NIE.CI,  b.NIE)
  names(out.put)=c("NDE", "deltalower95%CI","deltaupper95%CI", "bootlower95%CI","bootupper95%CI","NIE", "deltalower95%CI","deltaupper95%CI","bootlower95%CI","bootupper95%CI")
  return(out.put)
}


#Probit model to model outcome and continuous mediator----
probit.continuous.mediator.delta=function(data.set, confounders){
  data.set=as.data.frame(data.set)
  names(data.set)[c(1:3)]=c("Y", "S", "M")
  outcome.reg=glm(Y~., family = binomial (link="probit"), data=data.set)
  m.reg=lm(M~.-Y, data=data.set)
  beta.hat=summary(outcome.reg)$coefficients[,1]
  alpha.hat=summary(m.reg)$coefficients[,1]
  V.matrix=ONEcreate.vmatrix(outcome.reg, m.reg)
  total.effect=summary(glm(Y~.-M, family = binomial (link="probit"), data=data.set))$coefficients[,1]
  total.rd=pnorm(sum(total.effect))-pnorm(sum(total.effect[1])) #when S=1 compared to S=0
  total.rr=pnorm(sum(total.effect))/pnorm(sum(total.effect[1])) 
  
  A=ONEpoint.var.est.cont(1,0, beta.hat, alpha.hat, m.reg, confounders) #first part of difference
  B=ONEpoint.var.est.cont(0,0, beta.hat, alpha.hat, m.reg,confounders)
  C=ONEpoint.var.est.cont(1,1, beta.hat, alpha.hat, m.reg,confounders) #first part of difference
  pathprob1=B[1] #prob for path 0,0
  pathprob2=A[1]#prob for path 1,0
  pathprob3=C[1] #prob for path 1,1
  
  #NDE (1M0 - 0M0)
  path=A[1]-B[1] #point estimate of probability of this path
  der=matrix(A[-1]-B[-1], nrow=1)
  std.error=sqrt(der%*%V.matrix%*%t(der))
  results1=c(path, path-1.96*std.error,path+1.96*std.error) #s->y
  pval.rd1=2*min(pnorm(abs(path/std.error)),1-pnorm(abs(path/std.error)))
  path.rr=A[1]/B[1]
  RRder1=matrix((A[-1]/A[1])-(B[-1]/B[1]),nrow=1)
  RR.std.error=sqrt(RRder1%*%V.matrix%*%t(RRder1))
  RR1=c(path.rr, exp(log(path.rr)-1.96*RR.std.error) , exp(log(path.rr)+1.96*RR.std.error))
  pval.rr1=2*min(pnorm(abs(log(path.rr)/RR.std.error)),1-pnorm(abs(log(path.rr)/RR.std.error)))
  
  #NIE (1M1-1M0)
  path2=C[1]-A[1]
  der2=matrix(C[-1]-A[-1], nrow=1)
  std.error2=sqrt(der2%*%V.matrix%*%t(der2))
  results2=c(path2, path2-1.96*std.error2,path2+1.96*std.error2) #s-> g-> y
  pval.rd2=2*min(pnorm(abs(path2/std.error2)),1-pnorm(abs(path2/std.error2)))
  RRder2=matrix(C[-1]/C[1]-A[-1]/A[1], nrow=1)
  RR.std.error2=sqrt(RRder2%*%V.matrix%*%t(RRder2))
  RR2=c(C[1]/A[1], exp(log(C[1]/A[1])-1.96*RR.std.error2) , exp(log(C[1]/A[1])+1.96*RR.std.error2))
  pval.rr2=2*min(pnorm(abs(log(C[1]/A[1])/RR.std.error2)),1-pnorm(abs(log(C[1]/A[1])/RR.std.error2)))
  
  theoretical.rd=sum(results1[1], results2[1])
  theoretical.rr=prod(RR1[1], RR2[1])
  
  out.put=c(results1,pval.rd1,results2,pval.rd2,RR1,pval.rr1, RR2,pval.rr2, pathprob1,pathprob2,pathprob3)
  names(out.put)=c("RD NDE S->Y", "lower95%", "upper95%","pval1" ,"RD NIE S->G->Y", "lower95%", "upper95%","pval2","RR NDE S->Y", "lower95%", "upper95%","pval1RR","RR NIE S->G->Y", "lower95%", "upper95%","pval2RR","prob00","prob10","prob11")
  return(out.put)
}

probit.continuous.mediator.boot=function(data.set, confounders,n.boot){
  data.set=as.data.frame(data.set)
  names(data.set)[c(1:3)]=c("Y", "S", "M")
  outcome.reg=glm(Y~., family = binomial (link="probit"), data=data.set)
  m.reg=lm(M~.-Y, data=data.set)
  beta.hat=summary(outcome.reg)$coefficients[,1]
  alpha.hat=summary(m.reg)$coefficients[,1]
  total.effect=summary(glm(Y~.-M, family = binomial (link="probit"), data=data.set))$coefficients[,1]
  total.rd=pnorm(sum(total.effect))-pnorm(sum(total.effect[1])) #when S=1 compared to S=0
  total.rr=pnorm(sum(total.effect))/pnorm(sum(total.effect[1])) 
  
  A=ONEpoint.var.est.cont(1,0, beta.hat, alpha.hat, m.reg,confounders) #first part of difference
  B=ONEpoint.var.est.cont(0,0, beta.hat, alpha.hat, m.reg, confounders)
  C=ONEpoint.var.est.cont(1,1, beta.hat, alpha.hat, m.reg, confounders) #first part of difference
  
  var.boot=ONEcreate.var.boot.cont(n.boot, data.set, c())
  pathprob1=B[1] #prob for path 0,0
  pathprob2=A[1]#prob for path 1,0
  pathprob3=C[1] #prob for path 1,1
  
  #path 1
  path=A[1]-B[1] #point estimate of probability of this path
  results1=c(path) #s->y
  path.rr=A[1]/B[1]
  
  #path 2
  path2=C[1]-A[1]
  path.rr2=C[1]/A[1]
  
  
  b.RD1=c(quantile(var.boot[,1], probs=c(0.025, 0.975)))
  b.RD2=c(quantile(var.boot[,2], probs=c(0.025, 0.975)))
  b.RR1=c(quantile(var.boot[,3], probs=c(0.025, 0.975)))
  b.RR2=c(quantile(var.boot[,4], probs=c(0.025, 0.975)))
  
  total=dim(data.set)[1]
  #null hypothesis is 0 no effect in path
  valRD1=sum(var.boot[,1]>0)/n.boot
  pvalRD1=min(valRD1,(1-valRD1))*2
  valRD2=sum(var.boot[,2]>0)/n.boot
  pvalRD2=min(valRD2,(1-valRD2))*2
  
  #null hypothesis is log(1) no difference in relative risk. Doing it in the log scale as it may be less skewed
  valRR1=sum(log(var.boot[,3])>0)/n.boot
  pvalRR1=min(valRR1,(1-valRR1))*2
  valRR2=sum(log(var.boot[,4])>0)/n.boot
  pvalRR2=min(valRR2,(1-valRR2))*2
  
  out.put=c(path,b.RD1, pvalRD1,path2,b.RD2, pvalRD2, path.rr,b.RR1, pvalRR1, path.rr2,b.RR2,pvalRR2, pathprob1,pathprob2,pathprob3)
  names(out.put)=c("RD NDE S->Y", "bootlower95%", "bootupper95%","pval1", "RD NIE S->G->Y", "bootlower95%", "bootupper95%","pval2","RR NDE S->Y", "bootlower95%", "bootupper95%","RRpval1","RR NIE S->G->Y", "bootlower95%", "bootupper95%","RRpval2","prob00","prob10","prob11")
  return(out.put)
}

ONEpoint.var.est.cont=function(sa,sb,beta.hat, alpha.hat,m.reg,confounders){
  b0=beta.hat[1]
  bs=beta.hat[2]
  bm=beta.hat[3]
  beta.rest=beta.hat[-(1:3)]
  bX=beta.rest
  a0=alpha.hat[1]
  as=alpha.hat[2]
  alpha.rest=alpha.hat[-(1:2)]
  aX=alpha.rest
  #calc sigmas
  sigma.hat.m=sqrt(mean((m.reg$residuals)^2))
  sigma.star= sqrt((sigma.hat.m^2)*(bm^2) + 1 )  
  
  #############
  #the following needs to be changed to reflect potential interactions
  p=(b0+sum(bX*confounders)+bs*sa+bm*(a0+as*sb+sum(aX*confounders)))/sigma.star
  g.x=(b0+sum(bX*confounders)+bs*sa+bm*(a0+as*sb+sum(aX*confounders)))
  
  dB0= (exp(-(g.x^2)/(2*(sigma.star^2))))/(sqrt(2*pi)*sigma.star) #derivative beta 0
  dBs=dB0* sa #derivative beta s
  dBm= dB0*(sigma.star*(a0+as*sb+sum(aX*confounders))-((g.x/sigma.star)*(bm)*sigma.hat.m^2))/(sigma.star) #derivative beta m 
  
  da0=dB0*bm#derivative for alpha 0
  das=da0*sb#derivative for alpha s
  
  dBX=dB0*confounders
  daX=da0*confounders
  ######
  
  #combine all the dervatives into a vector, order based on the covariance matrices
  if (length(confounders)==0){
    derivatives=c( dB0, dBs, dBm, da0, das)
    return(c(pnorm(p), as.vector(derivatives)))
  }
  else{
    derivatives=c( dB0, dBs, dBm, dBX, da0, das,daX )
    return(c(pnorm(p), as.vector(derivatives)))
  }
  
}

ONEcreate.var.boot.cont=function(n.boot, sim.data1, confounders){
  var.boot1=matrix(NA, nr=n.boot, nc=4)
  s.size=dim(sim.data1)[1]
  for (boot.count in 1:n.boot){
    set.seed(37+boot.count*3)
    rv.normal.boot=sim.data1[sample(1:s.size, replace=TRUE),] #sampling same size as the data set
    b.outcome.reg=glm(Y~., family = binomial (link="probit"), data=rv.normal.boot)
    b.m.reg=lm(M~.-Y,  data=rv.normal.boot)
    b.beta.hat=summary(b.outcome.reg)$coefficients[,1]
    b.alpha.hat=summary(b.m.reg)$coefficients[,1]
    
    b.A=ONEpoint.var.est.cont(1,0, b.beta.hat, b.alpha.hat,  b.m.reg,confounders) #first part of difference
    b.B=ONEpoint.var.est.cont(0,0, b.beta.hat, b.alpha.hat,  b.m.reg,confounders)
    b.path=b.A[1]-b.B[1] #point estimate of probability of this path
    b.path.r=b.A[1]/b.B[1]
    b.C=ONEpoint.var.est.cont(1,1, b.beta.hat, b.alpha.hat, b.m.reg,confounders)
    b.path2=b.C[1]-b.A[1]
    b.path2.r=b.C[1]/b.A[1]
    
    var.boot1[boot.count,]=c(b.path, b.path2,  b.path.r, b.path2.r)
  }
  return(var.boot1)
}

#the following functions do not need to be changed
check=function(value, result.matrix){ 
  sum(result.matrix[,2]<value & result.matrix[,3]>value)
}

ONEcreate.vmatrix=function(o.reg,  m.reg1){
  beta.hat1=length(summary(o.reg)$coefficients[,1])
  alpha.hat1=length(summary(m.reg1)$coefficients[,1])
  
  dimensions=sum(beta.hat1+alpha.hat1)
  V.matrix.1=matrix(0, nr=dimensions, nc=dimensions)
  V.matrix.1[c(1:beta.hat1),c(1:beta.hat1)]=vcov(o.reg)
  V.matrix.1[c((1+beta.hat1):(beta.hat1+alpha.hat1)),c((1+beta.hat1):(beta.hat1+alpha.hat1))]=vcov(m.reg1)
  return (V.matrix.1)
}
